// ----------------------------------------------------------------------------
// XPlease.C
// A fido utility for Binkley Term to perform the following functions:
// - Create a poll file with different flavours for different domains
// ----------------------------------------------------------------------------
// List of errorlevels
// 001 - Config file not found
// 002 - Unable to create directory
// 003 - Unable to create pollfile
// 096 - command line paramter missing
// 097 - directory for given domain not found
// 098 - Illegal address sppecified
// 099 - Illegal command line
// ----------------------------------------------------------------------------
// Revision History:
// 950106:  fixed a really dumb where I created an .OLO file instead of a .FLO
//          one (sorry Hagen :-))
//          recompiled the source with Watcom C++ 10.0 and thus eben created
//          a DOS version for all those poor buggers out there :-)
// ----------------------------------------------------------------------------

// Include files

// #include <dir.h>     - Not needed for Watcom
#include <direct.h>
#include <dos.h>
#include <conio.h>
#include <ctype.h>
#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>

// defines

#define DO_POLL 0x01
#define DO_SEND 0x02
#define DO_RQST 0x03
#define DO_CHNG 0x04

#define FL_HOLD 0x00
#define FL_NORM 0x01
#define FL_DIRT 0x02
#define FL_CRSH 0x03

#define TRUE	1
#define FALSE	0

#define ILLEGAL 0xff

// global variables

typedef struct Domain {
  char  Name[9];                    // name of the domain
  char  Path[254];                  // path description
} Domain;

typedef struct naddress {
  int   zone,
        net,
        node,
        point;
  char  domain[9];
} naddress;

char linein[255];                   // needed for reading the config
naddress  dest;                     // destination address

Domain  dom_list[25];               // list of all available domains
int X_Action,
    X_Flavour,
    X_NumDom,
    X_MainAKA;

char  X_Outpath[254];               // outbound path for pollfile

int getaddress(char *, naddress *);
// ----------------------------------------------------------------------------
// display_help
// display the command syntax for XPlease
// ----------------------------------------------------------------------------

void display_help (void) {
     cputs ("\r\nInvalid command line detected!\r\n");
     cputs ("Calling syntax for XPlease:\r\n");
     cputs ("\r\n     XPlease <action> <flavour> <address>\r\n\r\n");
     cputs ("Where: <action>  can be POLL\r\n");
     cputs ("       <flavour> can be either CRASH, DIRECT, NORMAL, HOLD\r\n");
     cputs ("       <address> can be any {abbreviated} 5d-Fido-Address\r\n");
     cputs ("\r\n");
}

// ----------------------------------------------------------------------------
// do_it
// name tells enough, eh :-)
// ----------------------------------------------------------------------------

void do_it (void) {

  // we have:
  // the address in 'dest'
  // the path in X_Outpath
  // Main-AKA yes or no in X_MainAKA
  //
  // we need
  // the info whether we have a point here

  char temp[254];			// store the path in here
  char tm2[80];
  struct find_t FBLK;
  FILE	*handle;

  strcpy (temp, X_Outpath);
  if (temp[strlen(temp)-1] == '\\') {
      temp[strlen(temp)-1] = '\0';
  }

  if (X_MainAKA==FALSE) {                 // secondary aka -> add the zoneinfo
     sprintf(tm2, ".%03x", dest.zone);
     strcat (temp, tm2);
  }

  if (_dos_findfirst(temp, _A_SUBDIR, &FBLK) != 0) {	// outbound directory not found
    if (mkdir(temp) != 0) {
      cprintf ("Unable to create outbound directory %s !!\r\n", temp);
      cputs   ("Aborting ... (check your pathes)\r\n");
      exit (2);
    }
  }

  strcat (temp, "\\");

  if (dest.point != 0) {                  // want to poll a point
     sprintf (tm2, "%04x%04x.PNT", dest.net, dest.node);
     strcat (temp, tm2);
    if (_dos_findfirst(temp, _A_SUBDIR, &FBLK) != 0) {	// outbound directory not found
      if (mkdir(temp) != 0) {
        cprintf ("Unable to create outbound directory %s !!\r\n", temp);
        cputs   ("Aborting ... (check your pathes)\r\n");
        exit (2);
      }
    }
    sprintf (tm2, "0000%04x.", dest.point);
    strcat (temp, "\\");
  } else {
    sprintf (tm2, "%04x%04x.", dest.net, dest.node);
  }

  strcat (temp, tm2);

  switch (X_Flavour) {
    case FL_CRSH:  strcat (temp, "C"); break;
    case FL_NORM:  strcat (temp, "F"); break;
    case FL_DIRT:  strcat (temp, "D"); break;
    case FL_HOLD:  strcat (temp, "H"); break;
    default     :  strcat (temp, "O"); break;
  }

  strcat (temp, "LO");

  if (_dos_findfirst(temp, 0, &FBLK) == 0) {
    cprintf ("Done ... (file %s already exists)\r\n", temp);
    return;
  } else {
    handle = fopen(temp, "w");
    if (handle == NULL) {
      cprintf ("Error while creating pollfile %s\r\n", temp);
      exit (3);
    } else {
      cprintf ("Successfully created pollfile %s\r\n", temp);
    }
    fclose (handle);
  }
}

// ----------------------------------------------------------------------------
// search_domain
// looks for the given domain in the list of specified domains from the config
// ----------------------------------------------------------------------------

int search_domain (char *look) {

  int i;

  for (i=0;i<X_NumDom;i++) {
    if (strncmp(look, dom_list[i].Name, strlen(look)) == 0) {
      strcpy (X_Outpath, dom_list[i].Path);
      if (i==0) X_MainAKA = TRUE;
      return (TRUE);
    }
  }
  return (FALSE);
}

// ----------------------------------------------------------------------------
// display_flavour
// print the flavour of the request to the screen
// ----------------------------------------------------------------------------

void display_flavour (void) {

  switch (X_Flavour) {
	case FL_HOLD:	cprintf ("Hold (%d:%d/%d.%d@%s)\r\n",
					dest.zone, dest.net,
					dest.node, dest.point,
					dest.domain);   break;
	case FL_NORM:	cprintf ("Normal (%d:%d/%d.%d@%s)\r\n",
					dest.zone, dest.net,
					dest.node, dest.point,
					dest.domain	); break;
	case FL_DIRT:	cprintf ("Direct (%d:%d/%d.%d@%s)\r\n",
					dest.zone, dest.net,
					dest.node, dest.point,
					dest.domain	); break;
	case FL_CRSH:	cprintf ("Crash (%d:%d/%d.%d@%s)\r\n",
					dest.zone, dest.net,
					dest.node, dest.point,
					dest.domain	);  break;
	default:	cprintf ("ILLEGAL (%d:%d/%d.%d@%s)\r\n",
					dest.zone, dest.net,
					dest.node, dest.point,
					dest.domain	);break;  }
}

// ----------------------------------------------------------------------------
// parse_address
// parse the zone, net, node, point and domain information from a string
// ----------------------------------------------------------------------------

void parse_address (char *add) {

  if (getaddress (add, &dest) == 0) {
	cputs ("Illegal address detected!\r\n");
	cputs ("At least the <net>/<node> info is needed.\r\n");
	exit (98);
  }
  strupr (dest.domain);

}

// ----------------------------------------------------------------------------
// parse_action
// get the desired function from the command line
// ----------------------------------------------------------------------------

void parse_action (int cmds, char **action) {

  char temp[80];
  int  account;

  strcpy (temp, action[1]);
  strupr (temp);

  if (strncmp(temp, "POLL", strlen(temp)) == 0) {
     X_Action = DO_POLL;
     strcpy (temp, action[2]);
     strupr (temp);
     account = 3;
     if (strncmp(temp, "CRASH", strlen(temp)) == 0) {
	X_Flavour = FL_CRSH;
     } else {
	if (strncmp(temp, "HOLD", strlen(temp)) == 0) {
	   X_Flavour = FL_HOLD;
	} else {
	   if (strncmp(temp, "DIRECT", strlen(temp)) == 0) {
	      X_Flavour = FL_DIRT;
	   } else {
	      if (strncmp(temp, "NORMAL", strlen(temp)) == 0) {
		 X_Flavour = FL_NORM;
	      } else {
		 X_Flavour = FL_NORM;
		 account = 2;
	      }
	   }
	}
     }
     if (account > cmds) {
	cputs ("Error: Command line parameter missing!\r\n");
	cputs ("Check your command line and batch files.\r\n");
	exit  (96);
     }
     parse_address (action[account]);

  } else {
     X_Action = ILLEGAL;
  }

}

// ----------------------------------------------------------------------------
// parse_config
// parse the config file, get the different directories and the domainnames
// ----------------------------------------------------------------------------

void parse_config (char *path) {

  FILE *Ffile;
  char pfad[254];
  int	i=0;

  strcpy (pfad, path);
  strcat (pfad, "XPLEASE.CTL");

  Ffile = fopen (pfad, "rt");
  if (Ffile == NULL) {
     cputs ("Error while reading config file XPLEASE.CTL\r\n");
     cputs ("Make sure that you have a valid config file\r\n");
     exit  (1);
  }

  while ((fgets (linein, 250, Ffile) != NULL) && (i<25)) {
    if (linein[0] != ' '  &&
	linein[0] != ';'  &&
	linein[0] != ':'  &&
	linein[0] != '%'  &&
	linein[0] != '\r' &&
	linein[0] != '\n' &&
	linein[0] != 0) {
      sscanf (linein, "%s%s", dom_list[i].Name, dom_list[i].Path);
      strupr (dom_list[i].Name);
      strupr (dom_list[i].Path);
      i++;
    }
    X_NumDom = i;
  }
  fclose (Ffile);

}


void main (int argc, char *argv[]) {

  int i;
  char path[254];

//  clrscr();
  cputs ("\r\n\r\nXPlease v0.02 (C) 1993,95 Stefan Rubner 2:2480/10@fidonet\r\n");
  cputs ("An extended Please utility for Binkley-style mailers\r\n\r\n");

  X_MainAKA = FALSE;

  if (argc < 4) {
     display_help ();
     exit  (99);
  }

  i = strlen(argv[0])-1;
  for (;i>=0;i--) {
    if (argv[0][i] == '\\') break;
  };

  strncpy (path, argv[0], i+1);
  path[i+1] = 0;

  parse_action (argc, argv);
  if (X_Action == ILLEGAL) {
     display_help ();
     exit (99);
  }

  cputs ("Command/Flavour : ");
  switch (X_Action) {
	case DO_POLL:	cputs ("Poll / ");
			break;
	case DO_SEND:	cputs ("Send / ");
			break;
	case DO_RQST:	cputs ("Request / ");
			break;
	case DO_CHNG:	cputs ("Change / ");
			break;
	default:	cputs ("ILLEGAL / ");
			break;
  }

  display_flavour ();

  // -----
  // Read configuratoin file and get the domain-definitions
  // -----

  parse_config (path);

  if (dest.zone == 0) {			// we don't have a zone -> main-aka
     strcpy (X_Outpath, dom_list[0].Path);
     X_MainAKA = TRUE;
  } else {
     if (search_domain (dest.domain) == 0) {
	cputs ("ERROR!!\r\n");
	cputs ("Specified domain not found in internal list.\r\n");
	cputs ("Check your command line an XPlease.Ctl.\r\n");
	exit  (97);
     }
  }

  do_it ();

}




 /*
  *
  *  Extract address from string without wildcards
  *
  */

// static char *colon=":";
// static char *slash="/";


int getaddress(char *netnode, naddress *addr)
{
  char *p;
  char tmp[100];

  p=netnode;

  // -- strip leading spaces

  while (*p && (*p==' ')) p++;

  // -- empty address so fuck up!

  if(!*p)
  {
    addr->zone = 0;
    addr->net = 0;
    addr->node = 0;
    addr->point = 0;
    return (FALSE);
  }

  // -- strip trailing spaces
  //
  // shouldn't happen here for the argument is passed on the command line
  // so rem'd out :-)
  //
  //  i = cpos (' ', p);
  //  if (i)
  //  {
  //    strncpy(tmp, p, i);
  //    tmp[i - 1] = '\0';
  //    strcpy(p, &(p[i-1]));
  //  } else {
  strcpy(tmp, p);
  strcpy(p, "");
  //  }
  //
  p = tmp;


  /* If we have a zone (and the caller wants the zone to be passed back).. */

  if (strchr(tmp,':'))
  {
    addr->zone=atoi(p);
	// p=firstchar(p,colon,2);
	p = strchr(tmp, ':') + 1;
  }

  /* If we have a net number... */

  if (p && *p)
  {
    if (strchr(tmp,'/'))
    {
      addr->net=atoi(p);
	  // p=firstchar(p,slash,2);
	  p = strchr (tmp, '/') + 1;
    } else {
      return (FALSE);
    }
  }

  /* If we got a node number... */

  if (p && *p && *tmp!= '.') {
      addr->node=atoi(p);
  } else {
      return (FALSE);
  }

  if (p)
    while (*p && isdigit(*p))
	  p++;

  /* And finally check for a point number... */

  addr->point = 0;
  if (p && *p=='.')
  {
     p++;
     if (!p && *tmp=='.')
       p=tmp+1;
     if (p && *p) {
       addr->point=atoi(p);
     }
  }

  // Patched in domain - support (C) 1993 rbn

  addr->domain[0] = 0; 		// clear domain information

  if (p=strchr(tmp, '@')) {
      p++;
      if (p && *p) {
	strcpy(addr->domain, p);
      }
  }
  return (TRUE);
}


